Type: Package
Package: camtrapR
Title: Camera Trap Data Management and Analysis Framework
Version: 3.1.0
Date: 2026-07-25
Authors@R: c(
    person("Juergen", "Niedballa", email = "camtrapr@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-9187-2116")),
    person("Alexandre", "Courtiol", role = "aut",
           comment = c(ORCID = "0000-0003-0637-2959")),
    person("Rahel", "Sollmann", role = "aut",
           comment = c(ORCID = "0000-0002-1607-2039")),
    person("John", "Mathai", role = "ctb"),
    person("Seth Timothy", "Wong", role = "ctb",
           comment = c(ORCID = "0000-0001-8083-9268")),
    person("An The Truong", "Nguyen", role = "ctb",
           comment = c(ORCID = "0009-0000-2861-2672")),
    person("Azlan bin", "Mohamed", role = "ctb",
           comment = c(ORCID = "0000-0003-3788-4383")),
    person("Andrew", "Tilker", role = "ctb",
           comment = c(ORCID = "0000-0003-3630-8691")),
    person("Roshan", "Guharajan", role = "ctb",
           comment = c(ORCID = "0000-0001-8124-5461")),
    person("Ioannis", "Alexiou", role = "ctb",
           comment = c(ORCID = "0000-0001-5095-4767")),
    person("Andreas", "Wilting", role = c("ctb", "ths"),
           comment = c(ORCID = "0000-0001-5073-9186"))
  )
Description: Management and analysis of camera trap wildlife data through
    an integrated workflow. Provides functions for image/video
    organization and metadata extraction, species/individual
    identification. Creates detection histories for occupancy and spatial
    capture-recapture analyses, with support for multi-season studies.
    Includes tools for fitting community occupancy models in JAGS and
    NIMBLE, and an interactive dashboard for survey data visualization and
    analysis. Features visualization of species distributions and activity
    patterns, plus export capabilities for GIS and reports. Emphasizes
    automation and reproducibility while maintaining flexibility for
    different study designs.
License: GPL (>= 2)
URL: https://github.com/jniedballa/camtrapR,
        https://jniedballa.github.io/camtrapR/,
        https://groups.google.com/forum/#!forum/camtrapr
BugReports: https://groups.google.com/forum/#!forum/camtrapr
Depends: R (>= 4.1.0)
Imports: checkmate, crayon, data.table, dplyr, DT, generics, ggplot2,
        htmltools, leaflet, lubridate, methods, reshape2, secr, sf,
        shiny, shinyBS, shinydashboard, shinyjs, terra, tibble
Suggests: abind, bayesplot, callr, camtrapdp, coda, corrplot, elevatr,
        iNEXT, jsonify, jsonlite, knitr, lattice, magick, mapview,
        mockery, nimble, nimbleEcology, overlap, parallel, patchwork,
        pbapply, plotly, psych, R.rsp, raster, ritis, rjags, rlang,
        rmarkdown, RSQLite, rstudioapi, scales, shinytest2,
        shinyWidgets, stringr, taxize, tesseract, testthat, ubms,
        units, unmarked, viridisLite, withr, zip
VignetteBuilder: R.rsp
Encoding: UTF-8
SystemRequirements: ExifTool (https://exiftool.org/)
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-28 10:35:24 UTC; juergen
Author: Juergen Niedballa [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9187-2116>),
  Alexandre Courtiol [aut] (ORCID:
    <https://orcid.org/0000-0003-0637-2959>),
  Rahel Sollmann [aut] (ORCID: <https://orcid.org/0000-0002-1607-2039>),
  John Mathai [ctb],
  Seth Timothy Wong [ctb] (ORCID:
    <https://orcid.org/0000-0001-8083-9268>),
  An The Truong Nguyen [ctb] (ORCID:
    <https://orcid.org/0009-0000-2861-2672>),
  Azlan bin Mohamed [ctb] (ORCID:
    <https://orcid.org/0000-0003-3788-4383>),
  Andrew Tilker [ctb] (ORCID: <https://orcid.org/0000-0003-3630-8691>),
  Roshan Guharajan [ctb] (ORCID: <https://orcid.org/0000-0001-8124-5461>),
  Ioannis Alexiou [ctb] (ORCID: <https://orcid.org/0000-0001-5095-4767>),
  Andreas Wilting [ctb, ths] (ORCID:
    <https://orcid.org/0000-0001-5073-9186>)
Maintainer: Juergen Niedballa <camtrapr@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-28 11:20:02 UTC
Built: R 4.6.1; ; 2026-07-28 14:24:12 UTC; unix
