# Generated by roxygen2: do not edit by hand

S3method("[",camOp)
S3method("[",detHist)
S3method("[",records)
S3method(dplyr_reconstruct,records)
S3method(plot,camOp)
S3method(print,camOp)
S3method(print,cams)
S3method(print,cams_dp)
S3method(print,detHist)
S3method(print,records)
S3method(summary,camOp)
S3method(summary,records)
export(OCRdataFields)
export(PPC.community)
export(PPC.residuals)
export(activityDensity)
export(activityHistogram)
export(activityOverlap)
export(activityRadial)
export(addCopyrightTag)
export(addToPath)
export(aggregateStations)
export(appendSpeciesNames)
export(cameraOperation)
export(checkSpeciesIdentification)
export(checkSpeciesNames)
export(communityModel)
export(createCovariates)
export(createSpeciesFolders)
export(createStationFolders)
export(detectionHistory)
export(detectionMaps)
export(exifTagNames)
export(filterRecordTable)
export(fixDateTimeOriginal)
export(getSpeciesImages)
export(imageRename)
export(readCamtrapDP)
export(readWildlifeInsights)
export(recordTable)
export(recordTableIndividual)
export(simulateCamtrapData)
export(spatialDetectionHistory)
export(speciesAccum)
export(surveyDashboard)
export(surveyReport)
export(timeShiftImages)
export(writeDateTimeOriginal)
exportClasses(commOccu)
exportMethods(fit)
exportMethods(plot_coef)
exportMethods(plot_effects)
exportMethods(predict)
exportMethods(summary)
importFrom(DT,DTOutput)
importFrom(DT,datatable)
importFrom(DT,renderDT)
importFrom(checkmate,assert)
importFrom(checkmate,assert_character)
importFrom(checkmate,assert_choice)
importFrom(checkmate,assert_data_frame)
importFrom(checkmate,assert_logical)
importFrom(checkmate,checkClass)
importFrom(checkmate,makeAssertCollection)
importFrom(checkmate,reportAssertions)
importFrom(data.table,":=")
importFrom(data.table,foverlaps)
importFrom(data.table,rbindlist)
importFrom(data.table,setDF)
importFrom(data.table,setDT)
importFrom(data.table,setkey)
importFrom(dplyr,"%>%")
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,count)
importFrom(dplyr,distinct)
importFrom(dplyr,dplyr_reconstruct)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,n_distinct)
importFrom(dplyr,pull)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,summarize)
importFrom(dplyr,sym)
importFrom(generics,fit)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,expr)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_linerange)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_pointrange)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,geom_violin)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,median_hilow)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,stat_summary)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_void)
importFrom(ggplot2,vars)
importFrom(ggplot2,xlab)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylab)
importFrom(ggplot2,ylim)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.off)
importFrom(grDevices,extendrange)
importFrom(grDevices,hcl.colors)
importFrom(grDevices,heat.colors)
importFrom(grDevices,png)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,grconvertX)
importFrom(graphics,grconvertY)
importFrom(graphics,hist)
importFrom(graphics,image)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,pairs)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,plot.default)
importFrom(graphics,plot.new)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,rect)
importFrom(graphics,segments)
importFrom(graphics,strheight)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(leaflet,addCircleMarkers)
importFrom(leaflet,addLayersControl)
importFrom(leaflet,addPolygons)
importFrom(leaflet,addTiles)
importFrom(leaflet,clearGroup)
importFrom(leaflet,layersControlOptions)
importFrom(leaflet,leaflet)
importFrom(leaflet,leafletOutput)
importFrom(leaflet,leafletProxy)
importFrom(leaflet,renderLeaflet)
importFrom(lubridate,"%within%")
importFrom(lubridate,as_date)
importFrom(lubridate,as_datetime)
importFrom(lubridate,days)
importFrom(lubridate,ddays)
importFrom(lubridate,dhours)
importFrom(lubridate,dseconds)
importFrom(lubridate,hour)
importFrom(lubridate,int_end)
importFrom(lubridate,int_overlaps)
importFrom(lubridate,int_start)
importFrom(lubridate,interval)
importFrom(lubridate,is.Date)
importFrom(lubridate,is.POSIXt)
importFrom(lubridate,parse_date_time)
importFrom(lubridate,stamp)
importFrom(lubridate,time_length)
importFrom(lubridate,tz)
importFrom(lubridate,ymd_hms)
importFrom(methods,.hasSlot)
importFrom(methods,hasArg)
importFrom(methods,is)
importFrom(methods,new)
importFrom(reshape2,melt)
importFrom(secr,RMarkInput)
importFrom(secr,covariates)
importFrom(secr,make.capthist)
importFrom(secr,read.traps)
importFrom(secr,traps)
importFrom(secr,usage)
importFrom(sf,st_as_sf)
importFrom(sf,st_bbox)
importFrom(sf,st_buffer)
importFrom(sf,st_convex_hull)
importFrom(sf,st_coordinates)
importFrom(sf,st_drop_geometry)
importFrom(sf,st_intersection)
importFrom(sf,st_make_valid)
importFrom(sf,st_set_crs)
importFrom(sf,st_transform)
importFrom(sf,st_union)
importFrom(sf,st_write)
importFrom(shiny,HTML)
importFrom(shiny,actionButton)
importFrom(shiny,checkboxGroupInput)
importFrom(shiny,checkboxInput)
importFrom(shiny,column)
importFrom(shiny,conditionalPanel)
importFrom(shiny,dateRangeInput)
importFrom(shiny,div)
importFrom(shiny,fileInput)
importFrom(shiny,fluidRow)
importFrom(shiny,h1)
importFrom(shiny,h2)
importFrom(shiny,h3)
importFrom(shiny,h4)
importFrom(shiny,h5)
importFrom(shiny,helpText)
importFrom(shiny,hr)
importFrom(shiny,htmlOutput)
importFrom(shiny,icon)
importFrom(shiny,incProgress)
importFrom(shiny,isolate)
importFrom(shiny,modalButton)
importFrom(shiny,modalDialog)
importFrom(shiny,need)
importFrom(shiny,numericInput)
importFrom(shiny,observe)
importFrom(shiny,observeEvent)
importFrom(shiny,outputOptions)
importFrom(shiny,plotOutput)
importFrom(shiny,radioButtons)
importFrom(shiny,reactive)
importFrom(shiny,reactiveTimer)
importFrom(shiny,reactiveVal)
importFrom(shiny,reactiveValues)
importFrom(shiny,removeModal)
importFrom(shiny,removeNotification)
importFrom(shiny,renderPlot)
importFrom(shiny,renderPrint)
importFrom(shiny,renderTable)
importFrom(shiny,renderText)
importFrom(shiny,renderUI)
importFrom(shiny,req)
importFrom(shiny,selectInput)
importFrom(shiny,showModal)
importFrom(shiny,showNotification)
importFrom(shiny,sliderInput)
importFrom(shiny,span)
importFrom(shiny,tabPanel)
importFrom(shiny,tableOutput)
importFrom(shiny,tabsetPanel)
importFrom(shiny,tagList)
importFrom(shiny,tags)
importFrom(shiny,textInput)
importFrom(shiny,textOutput)
importFrom(shiny,uiOutput)
importFrom(shiny,updateActionButton)
importFrom(shiny,updateCheckboxGroupInput)
importFrom(shiny,updateCheckboxInput)
importFrom(shiny,updateDateRangeInput)
importFrom(shiny,updateNumericInput)
importFrom(shiny,updateSelectInput)
importFrom(shiny,updateSelectizeInput)
importFrom(shiny,updateSliderInput)
importFrom(shiny,updateTextInput)
importFrom(shiny,validate)
importFrom(shiny,varSelectizeInput)
importFrom(shiny,verbatimTextOutput)
importFrom(shiny,wellPanel)
importFrom(shiny,withProgress)
importFrom(shinyBS,bsTooltip)
importFrom(shinydashboard,dropdownMenu)
importFrom(shinydashboard,dropdownMenuOutput)
importFrom(shinydashboard,renderMenu)
importFrom(stats,aggregate)
importFrom(stats,dbinom)
importFrom(stats,dpois)
importFrom(stats,end)
importFrom(stats,na.omit)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,rmultinom)
importFrom(stats,rnorm)
importFrom(stats,rpois)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,start)
importFrom(stats,window)
importFrom(terra,"values<-")
importFrom(terra,mask)
importFrom(terra,nlyr)
importFrom(terra,project)
importFrom(terra,rast)
importFrom(terra,resample)
importFrom(terra,vect)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,menu)
importFrom(utils,modifyList)
importFrom(utils,object.size)
importFrom(utils,read.csv)
importFrom(utils,read.table)
importFrom(utils,sessionInfo)
importFrom(utils,str)
importFrom(utils,unzip)
importFrom(utils,write.csv)
importFrom(utils,zip)
