globalVariables(c(
  "PAO", "Observed", "Predicted", "Records", "Species", "Stations",
  "covariateValue", "lower", "upper", "median",
  "modelEffects",
  "generateFormula",
  "export_table",
  "plot_effect_det",
  "plot_effect_occu",
  "generate_spatial_predictions",
  "create_species_detections_sf",
  "all_models",
  "p",
  ".data",
  
  # speciesAccum:
  "survey_date",  "survey_day",  "survey_day_by_station", "station_setup_date"
  
  
))